/*
	desc      : wkhtmltox example
					this is a simple example program showing how to use the wkhtmltox pdf bindings 
	author    : raphael smart
	date      : august 2012

	licence/disclaimer:

	This software is provided 'as-is', without any express or implied warranty.
	In no event will the author be held liable for any damages arising from
	the use of this software.

	other license:

	www.mingw.org

		bin/libgcc_s_dw2-1.dll
		bin/mingwm10.dll

	openssl

		bin/libeay32.dll
		bin/ssleay32.dll

	code.google.com/p/wkhtmltopdf

		libwkhtmltox.dll
		libwkhtmltox.so
		settings.txt
*/
namespace pdftest
constant __APPNAME__ = "pdftest"

include std/io.e

include include/wkhtmltox.e as wkhtml

enum

	false = 0,
	true

--// create pdf
procedure makepdf(sequence pageurl, sequence pdfname="output.pdf", sequence msg="")

	if length(msg) then
		puts(STDOUT, "---- " & msg & "\n")
	end if

	/*
	   create a global settings object used to store options that are not
	   related to input objects, note that control of this object is parsed to
	   the converter later, which is then responsible for freeing it
	*/
	atom glob = wkhtml:create_global_settings()

	--// create the actual converter object used to convert the webpage 
	atom converter = wkhtml:create_converter(glob)

	/*
	   create settings that are used to store information
	   related to an input object. these objects are passed to
	   the converter which is then responsible for freeing them
	*/
	for i = 1 to length(pageurl) do
		printf(STDOUT, "add page url '%s'\n", { pageurl[i] })
		--// init page
		atom page = wkhtml:create_object_settings()
		wkhtml:set_object_setting(page, "page", pageurl[i])
		--// enable javascript
		wkhtml:set_object_setting(page, "web.enableJavascript", "true")
		--// enable plugins
		wkhtml:set_object_setting(page, "web.enablePlugins", "true")
		--// add page to generator
		wkhtml:add_object(converter, page, "")
	end for

	--// where do we want the pdf output to be saved 
	wkhtml:set_global_setting(glob, "out", pdfname)

	--// perform the actual conversion
	if (not wkhtml:convert(converter)) then
		printf(STDERR, "conversion failed!\n")
	end if

	/* 
		output possible http error code encountered

		note that some web pages provide a 404 error (page not found)
		but the pdf file is generated as expected
	*/

	printf(STDOUT, "httpErrorCode: %d\n", { wkhtml:http_error_code(converter) })
	printf(STDOUT, "---- Done\n\n")

	--// destroy the converter object since we are done with it
	wkhtml:destroy_converter(converter)

end procedure

--// main

	sequence argv = command_line()
	integer argc  = length(argv)

	sequence pageurl = 
	{
		"http://www.google.com/news",
		"http://www.bbc.co.uk/news"
	}

	--// init wkhtmltox
	wkhtml:init(true)

	--// version number
	printf(STDOUT, "---- wkhtmltox version : %s\n\n", { wkhtml:version() })

	if argc > 2 then

		--// generate pdf from user supplied url

		sequence out = "page_user.pdf"
		if argc > 3 and length(argv[4]) then
			out = argv[4]
		end if

		makepdf({ argv[3] }, out, "Generating user pdf")

	else

		--// generate test pdf's

		--// generate pdf from local pages on disk
		makepdf({ "test/test.html" },      "page_local.pdf",       "Generating pdf from local file")
		makepdf({ "test/testscript.html"}, "page_localscript.pdf", "Generating pdf from local file with embedded javascript")

		--// remote single page
		makepdf({ pageurl[$] }, "page_single.pdf", "Generating single page pdf")

		--// remote multiple pages
		makepdf(pageurl, "page_multi.pdf", "Generating multi-page pdf")

		--// remote single page (https)
		makepdf({ "https://www.yahoo.com" }, "page_https.pdf", "Generating https pdf")

	end if

	--// cleanup
	wkhtml:deinit()
